#include <avr/io.h>
#include <avr/interrupt.h>  /* for sei() */
#include <util/delay.h>     /* for _delay_ms() */
#include <avr/pgmspace.h>   /* required by usbdrv.h */
#include "usbdrv.h"

#define VENDOR_RQ_SET_LED    0
#define VENDOR_RQ_GET_SWITCH 1
#define VENDOR_RQ_GET_LIGHT1  2
#define VENDOR_RQ_GET_LIGHT2  3
#define VENDOR_RQ_GET_LIGHT3  4
#define VENDOR_RQ_GET_LIGHT4  5

/* ------------------------------------------------------------------------- */
/* ---------------------------- Utility Functions -------------------------- */
/* ------------------------------------------------------------------------- */

uint8_t in_c(uint8_t pin)
{
    // read logic level of port-C's pin
    DDRC &= ~(1<<pin);
    return (PINC >>pin) & 1;
} 

void out_c(uint8_t pin, uint8_t val)
{
    // write logic level to port-C's pin
    DDRC |= 1<<pin;
    if (val)
        PORTC |= 1<<pin;
    else
        PORTC &= ~(1<<pin);
}

uint16_t read_adc(uint8_t channel)
{
    ADMUX = 0b01000000;
    ADMUX |= channel;
    ADCSRA = 0b11000110;

    while ((ADCSRA & (1<<ADSC)))
        ;
    return ADCL + ADCH*256;
}

/* ------------------------------------------------------------------------- */
/* ----------------------------- USB interface ----------------------------- */
/* ------------------------------------------------------------------------- */
uint16_t A;
usbMsgLen_t usbFunctionSetup(uint8_t data[8])
{
    usbRequest_t *rq = (void *)data;
    static uint8_t dataBuffer[2];  /* must stay valid when usbFunctionSetup returns */

    /*if (rq->bRequest == VENDOR_RQ_SET_LED)
    {
        uint8_t led_no  = rq->wValue.bytes[0];
        uint8_t led_val = rq->wValue.bytes[1];
        out_c(led_no, led_val);
        return 0;
    }

    else if (rq->bRequest == VENDOR_RQ_GET_SWITCH)
    {
        dataBuffer[0] = !in_c(3);
        usbMsgPtr = dataBuffer;
        return 1;
    }*/

    if (rq->bRequest == VENDOR_RQ_GET_LIGHT1)
    {        
        A = read_adc(0);        
        dataBuffer[0] = A;
        dataBuffer[1] = A>>8;
        usbMsgPtr = dataBuffer;
        return 2;
    }
    
    else if (rq->bRequest == VENDOR_RQ_GET_LIGHT2)
    {        
        A = read_adc(1);        
        dataBuffer[0] = A;
        dataBuffer[1] = A>>8;
        usbMsgPtr = dataBuffer;
        return 2;
    }

    else if (rq->bRequest == VENDOR_RQ_GET_LIGHT3)
    {        
        A = read_adc(2);        
        dataBuffer[0] = A;
        dataBuffer[1] = A>>8;
        usbMsgPtr = dataBuffer;
        return 2;
    }
    else if (rq->bRequest == VENDOR_RQ_GET_LIGHT4)
    {        
        A = read_adc(3);        
        dataBuffer[0] = A;
        dataBuffer[1] = A>>8;
        usbMsgPtr = dataBuffer;
        return 2;
    }
    return 0;   /* default for not implemented requests: return no data back to host */
}

/* ------------------------------------------------------------------------- */



int main(void)
{
    DDRC  = 0b00000000; // PC0..PC2 = Output, PC3..PC5 = Input
    //PORTC = (1 << PC3);                 // Enable pull-up at PC3

    usbInit();
    usbDeviceDisconnect();  /* enforce re-enumeration, do this while interrupts are disabled! */
    _delay_ms(300);
    usbDeviceConnect();
    sei();
    for(;;){                /* main event loop */
        usbPoll();
    }
    return 0;
}

/* ------------------------------------------------------------------------- */
